;nyquist plug-in
;version 3
;type process
;preview linear
;name "Phonic Mind"
;action "Reducing center-panned vocals..."
;info "For reducing center-panned vocals"
;author "Kaiki Ogata"
;copyright "Released under terms of the GNU General Public License version 2"

;; Phonic Mind, created (or combined) by Kaiki Ogata
;; Based on Vocal Remover, High Pass Filter, and Low Pass Filter

;control action "Remove vocals or view Help" choice "Reduce vocals,View Help" 0
;control band-choice "Removal choice" choice "Entire spectrum,Remove frequency band,Retain frequency band" 0
;control low-range "Cutoff high frequencies" string " " "500"
;control high-range "Cutoff low frequencies (Hz)" string " " "2000"

; Initialize globals
(setf message "")     ; empty output message

(defun help ()
  (format nil
"Phonic Mind, named after the most popular vocal remover online, is basically my own modification of Vocal Remover that's basically programmed and copied and pasted in poor Nyquist. That's all. Try figuring out the controls yourself, because it's just Vocal Remover dumbed down to as low as possible. Yes, I'm not a Nyquist programmer."))

(defun string-to-list (str)
  (read (make-string-input-stream (format nil "(~a)" str))))

;;; ERROR CHECKING:

;; Check that selected audio is stereo
(defun check-stereo ()
  (if (soundp s)
      (setf message (format nil
                      "~%Phonic Mind requires an unsplit, stereo track.~%~

                      If you have a stereo track split into left and right~%~
                      channels, use 'Make Stereo Track' on the Track~%~
                      Drop-Down Menu, then run Vocal Remover again.~%"))))

;;; Check that frequency range is valid
(defun check-range ()
  (setq low-range (first (string-to-list low-range)))
  (setq high-range (first (string-to-list high-range)))
  (if (or (not (numberp low-range))
          (not (numberp high-range))
          (< low-range 0)
          (> low-range 24000)
          (< high-range 0)
          (> high-range 24000))
      (progn
        (if (= band-choice 1)
            (setf band-type "remove")
            (setf band-type "retain"))
        (setf message (format nil
                        "~a~%~
                        Enter both a lower and an upper limit for the~%~
                        frequency band you want to ~a.~%~
                        You entered: \"~a\"   \"~a\"~%~%~
                        Both values must be between 0 and 20000."
                        message
                        band-type
                        low-range
                        high-range)))
      ;; Else ensure that high-range > low-range.
      (let ((temp low-range))
        (when (> low-range high-range)
          (setq low-range high-range)
          (setq high-range temp))))
  ;; Range values must cannot be higher than Nyquist frequency.
  (setq low-range (min low-range (/ *sound-srate* 2)))
  (setq high-range (min high-range (/ *sound-srate* 2))))


(defun show-message ()
  ;; output to both message box and to debug window
  ;; Copying from debug window is supported on all platforms.
  (when (= action 0)      ; error
    (setf message (format nil "Error.~%~a" message)))
  (format t message)
  (format nil message))


;;; DSP FUNCTIONS:

(defun bandpass (sig low high)
  (lowpass8
    (highpass8 sig low)
    high))

(defun bandstop (sig low high)
  (sum (lowpass8 sig low)
       (highpass8 sig high)))

(defun CentrePanRemove ()
  (cond
    ((= band-choice 1)            ; remove frequencies inside range
      (sum (aref s 0)
           (mult -1 (aref s 1))
           (bandstop (aref s 1) low-range high-range)))
    ((= band-choice 2)            ; remove frequencies outside range
      (sum (aref s 0)
           (mult -1 (aref s 1))
           (bandpass (aref s 1) low-range high-range)))
    (t                            ; invert and add right to left channel
      (sum (aref s 0)
           (mult -1 (aref s 1))))))


;;; MAIN PROGRAM:

(cond
  ((= action 1)       ; Show help
    (setf message (help)))
  ((= band-choice 0)  ; Remove full spectrum
    (check-stereo))
  (t                  ; Remove band limited
    (check-stereo)
    (check-range)))

(if (= (length message) 0)
    (CentrePanRemove)
    (show-message))
